#!/usr/bin/env python

"""
weatherreceiver.py - Demonstrates how to use XQuery from Python.
                     See http://www.yr.no/verdata/1.3321307 for the terms of
                     use of the yr.no weather service.

Copyright (C) 2010 Nokia Corporation and/or its subsidiary(-ies).
All rights reserved.
Contact: Nokia Corporation (qt-info@nokia.com)

This file is part of the documentation of Qt. It was originally
published as part of Qt Quarterly.

$QT_BEGIN_LICENSE:LGPL$
Commercial Usage
Licensees holding valid Qt Commercial licenses may use this file in
accordance with the Qt Commercial License Agreement provided with the
Software or, alternatively, in accordance with the terms contained in
a written agreement between you and Nokia.

GNU Lesser General Public License Usage
Alternatively, this file may be used under the terms of the GNU Lesser
General Public License version 2.1 as published by the Free Software
Foundation and appearing in the file LICENSE.LGPL included in the
packaging of this file.  Please review the following information to
ensure the GNU Lesser General Public License version 2.1 requirements
will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.

In addition, as a special exception, Nokia gives you certain additional
rights.  These rights are described in the Nokia Qt LGPL Exception
version 1.1, included in the file LGPL_EXCEPTION.txt in this package.

GNU General Public License Usage
Alternatively, this file may be used under the terms of the GNU
General Public License version 3.0 as published by the Free Software
Foundation and appearing in the file LICENSE.GPL included in the
packaging of this file.  Please review the following information to
ensure the GNU General Public License version 3.0 requirements will be
met: http://www.gnu.org/copyleft/gpl.html.

If you have questions regarding the use of this file, please contact
Nokia at qt-info@nokia.com.
$QT_END_LICENSE$
"""

import os, sys, time
from PyQt4.QtCore import QDate, QDateTime, QFile, QSize, QString, Qt, \
                         QVariant, SIGNAL
from PyQt4.QtGui import QApplication, QCheckBox, QComboBox, QFormLayout, \
                        QIcon, QLabel, QTableWidget, QTableWidgetItem, \
                        QVBoxLayout, QWidget
from PyQt4.QtXmlPatterns import QAbstractXmlReceiver, QXmlItem, QXmlQuery

try:
    import weatherreceiver_rc
except ImportError:
    sys.stderr.write(
        "Please process the weatherreceiver.qrc file before running this program. "
        "For example with:\n\n"
        "pyrcc4 -o weatherreceiver_rc.py weatherreceiver.qrc\n\n"
        )
    sys.exit(1)

query_string = """declare variable $url external;

doc($url)//sun,
for $time in doc($url)//tabular/time
    order by $time/@from
    return <time>
        { $time/@from }
        <symbol>
            { $time/symbol/@name, $time/symbol/@number }
        </symbol>
        <temperature>
            { $time/temperature/@value, $time/temperature/@unit }
        </temperature>
        </time>
"""

class Receiver(QAbstractXmlReceiver):

    forecast_elements = ("symbol", "temperature")
    
    def __init__(self, namePool):
    
        QAbstractXmlReceiver.__init__(self)
        self.namePool = namePool
        self.sun = {}
        self.forecasts = []
        self._stack = []
        self._current_forecast = {}
    
    def attribute(self, xmlname, value):
    
        name = unicode(xmlname.localName(self.namePool))
        value = unicode(value.toString())
        current_element = self._stack[-1]
        
        if current_element == "sun":
            self.sun[name] = value
        
        elif current_element == "time":
            self._current_forecast[name] = value
        
        elif current_element in self.forecast_elements:
            d = self._current_forecast.setdefault(current_element, {})
            d[name] = value
    
    def characters(self, value):
        pass
    
    def comment(self, value):
        pass
    
    def namespaceBinding(self, xmlname):
        pass
    
    def startElement(self, xmlname):
        name = xmlname.localName(self.namePool)
        self._stack.append(unicode(name))
        
        if name == "time":
            self._current_forecast = {}
    
    def endElement(self):
        if self._stack.pop() == "time":
            self.forecasts.append(self._current_forecast)
    
    def startOfSequence(self):
        pass
    
    def endOfSequence(self):
        pass


class WeatherWidget(QWidget):

    def __init__(self, parent = None):
    
        QWidget.__init__(self, parent)
        
        self.recentQueries = {
            "Italia/Toscana/Firenze": None,
            "Norway/Oslo/Oslo/Oslo": None,
            "Tyskland/Berlin/Berlin": None
            }
        
        self.queryTimes = {
            "Italia/Toscana/Firenze": 0.0,
            "Norway/Oslo/Oslo/Oslo": 0.0,
            "Tyskland/Berlin/Berlin": 0.0
            }
        
        placeCombo = QComboBox()
        placeCombo.addItem("Italia/Toscana/Firenze")
        placeCombo.addItem("Norway/Oslo/Oslo/Oslo")
        placeCombo.addItem("Tyskland/Berlin/Berlin")
        
        self.todayCheckBox = QCheckBox(self.tr("&Only today"))
        
        self.forecastTable = QTableWidget()
        self.forecastTable.setColumnCount(3)
        self.forecastTable.horizontalHeader().hide()
        self.forecastTable.verticalHeader().hide()
        self.forecastTable.horizontalHeader().setStretchLastSection(True)
        self.forecastTable.setIconSize(QSize(50, 50))
        
        creditLabel = QLabel(self.tr("Weather forecast from yr.no, delivered "
                                     "by the Norwegian Meteorological "
                                     "Institute and the NRK"))
        creditLabel.setWordWrap(True)
        
        self.connect(placeCombo, SIGNAL("currentIndexChanged(QString)"),
                     self.fetchForecast)
        self.connect(self.todayCheckBox, SIGNAL("toggled(bool)"),
                     self.updateTable)
        
        layout = QFormLayout()
        layout.addRow(self.tr("&Place:"), placeCombo)
        layout.addRow(self.tr("&Forecast:"), self.todayCheckBox)
        
        mainLayout = QVBoxLayout(self)
        mainLayout.addLayout(layout)
        mainLayout.addWidget(self.forecastTable)
        mainLayout.addWidget(creditLabel)
        
        self.setWindowTitle(self.tr("Weather Forecast"))
        self.fetchForecast("Italia/Toscana/Firenze")
    
    def fetchForecast(self, place):
    
        now = time.time()
        place = unicode(place)
        
        if now - self.queryTimes[place] >= 600:
            pass
        else:
            self.receiver = self.recentQueries[place]
            self.updateTable()
            return
        
        url = "http://www.yr.no/place/" + place + "/forecast.xml"
        
        query = QXmlQuery()
        query.bindVariable("url", QXmlItem(QVariant(url)))
        query.setQuery(query_string)
        
        if query.isValid():
        
            self.receiver = Receiver(query.namePool())
            query.evaluateTo(self.receiver)
            self.recentQueries[place] = self.receiver
            self.queryTimes[place] = now
            self.updateTable()
    
    def updateTable(self):
    
        if "rise" in self.receiver.sun and "set" in self.receiver.sun and \
            len(self.receiver.forecasts) > 0:
        
            self.forecastTable.clear()
            self.forecastTable.setRowCount(0)
            
            sunrise = QDateTime.fromString(self.receiver.sun["rise"], Qt.ISODate)
            sunset = QDateTime.fromString(self.receiver.sun["set"], Qt.ISODate)
            
            for forecast in self.receiver.forecasts:
            
                datetime = QDateTime.fromString(forecast["from"], Qt.ISODate)
                onlyToday = self.todayCheckBox.checkState() == Qt.Checked
                
                if not onlyToday or datetime.date() == QDate.currentDate():
                
                    description = forecast["symbol"]["name"]
                    identifier = forecast["symbol"]["number"]
                    temperature = forecast["temperature"]["value"]
                    unit = forecast["temperature"]["unit"]
                    if unit == u"celcius":
                        unit = u"\u00b0C"
                    
                    row = self.forecastTable.rowCount()
                    self.forecastTable.insertRow(row)
                    
                    if onlyToday:
                        timeItem = QTableWidgetItem(datetime.time().toString())
                    else:
                        timeItem = QTableWidgetItem(datetime.toString("dddd\nhh:mm"))
                    timeItem.setFlags(Qt.ItemIsEnabled)
                    self.forecastTable.setItem(row, 0, timeItem)
                    
                    if sunrise.time() <= datetime.time() < sunset.time():
                        symbol = ":images/%02id.png" % int(identifier)
                    else:
                        symbol = ":images/%02in.png" % int(identifier)
                    
                    fallback = ":images/%02i.png" % int(identifier)
                    
                    if QFile.exists(symbol):
                        symbolItem = QTableWidgetItem(QIcon(symbol), description)
                    elif QFile.exists(fallback):
                        symbolItem = QTableWidgetItem(QIcon(fallback), description)
                    else:
                        symbolItem = QTableWidgetItem(description)
                    
                    symbolItem.setFlags(Qt.ItemIsEnabled)
                    
                    self.forecastTable.setItem(row, 1, symbolItem)
                    
                    temperatureItem = QTableWidgetItem(temperature + u" " + unit)
                    temperatureItem.setFlags(Qt.ItemIsEnabled)
                    self.forecastTable.setItem(row, 2, temperatureItem)
            
            self.forecastTable.resizeColumnsToContents()
            self.forecastTable.resizeRowsToContents()


if __name__ == "__main__":

    app = QApplication(sys.argv)
    
    window = WeatherWidget()
    window.show()
    
    sys.exit(app.exec_())
