/****************************************************************************
**
** Copyright (C) 2010 Nokia Corporation and/or its subsidiary(-ies).
** Contact: Nokia Corporation (qt-info@nokia.com)
**
** This file is part of the documentation of Qt. It was originally
** published as part of Qt Quarterly.
**
** $QT_BEGIN_LICENSE:LGPL$
** Commercial Usage
** Licensees holding valid Qt Commercial licenses may use this file in
** accordance with the Qt Commercial License Agreement provided with the
** Software or, alternatively, in accordance with the terms contained in
** a written agreement between you and Nokia.
**
** GNU Lesser General Public License Usage
** Alternatively, this file may be used under the terms of the GNU Lesser
** General Public License version 2.1 as published by the Free Software
** Foundation and appearing in the file LICENSE.LGPL included in the
** packaging of this file.  Please review the following information to
** ensure the GNU Lesser General Public License version 2.1 requirements
** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
**
** In addition, as a special exception, Nokia gives you certain additional
** rights.  These rights are described in the Nokia Qt LGPL Exception
** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
**
** GNU General Public License Usage
** Alternatively, this file may be used under the terms of the GNU
** General Public License version 3.0 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.  Please review the following information to
** ensure the GNU General Public License version 3.0 requirements will be
** met: http://www.gnu.org/copyleft/gpl.html.
**
** If you have questions regarding the use of this file, please contact
** Nokia at qt-info@nokia.com.
** $QT_END_LICENSE$
**
****************************************************************************/

#include <QtGui>
#include <QXmlQuery>

#include "receiver.h"
#include "weatherwidget.h"

#define query_string "declare variable $url external;\n" \
                     "\n" \
                     "doc($url)//sun,\n" \
                     "for $time in doc($url)//tabular/time\n" \
                     "    order by $time/@from\n" \
                     "    return <time>\n" \
                     "       { $time/@from }\n" \
                     "       <symbol>\n" \
                     "           { $time/symbol/@name, $time/symbol/@number }\n" \
                     "       </symbol>\n" \
                     "       <temperature>\n" \
                     "           { $time/temperature/@value, $time/temperature/@unit }\n" \
                     "       </temperature>\n" \
                     "       </time>\n"

WeatherWidget::WeatherWidget(QWidget *parent)
    : QWidget(parent), receiver(0)
{
    recentQueries["Italia/Toscana/Firenze"] = 0;
    recentQueries["Norway/Oslo/Oslo/Oslo"] = 0;
    recentQueries["Tyskland/Berlin/Berlin"] = 0;

    queryTimes["Italia/Toscana/Firenze"] = 0;
    queryTimes["Norway/Oslo/Oslo/Oslo"] = 0;
    queryTimes["Tyskland/Berlin/Berlin"] = 0;

    QComboBox *placeCombo = new QComboBox();
    placeCombo->addItem("Italia/Toscana/Firenze");
    placeCombo->addItem("Norway/Oslo/Oslo/Oslo");
    placeCombo->addItem("Tyskland/Berlin/Berlin");

    todayCheckBox = new QCheckBox(tr("&Only today"));

    forecastTable = new QTableWidget();
    forecastTable->setColumnCount(3);
    forecastTable->horizontalHeader()->hide();
    forecastTable->verticalHeader()->hide();
    forecastTable->horizontalHeader()->setStretchLastSection(true);
    forecastTable->setIconSize(QSize(50, 50));

    QLabel *creditLabel = new QLabel(tr("Weather forecast from yr.no, delivered "
                                        "by the Norwegian Meteorological "
                                        "Institute and the NRK"));
    creditLabel->setWordWrap(true);

    connect(placeCombo, SIGNAL(currentIndexChanged(QString)),
            this, SLOT(fetchForecast(QString)));
    connect(todayCheckBox, SIGNAL(toggled(bool)), this, SLOT(updateTable()));

    QFormLayout *layout = new QFormLayout();
    layout->addRow(tr("&Place:"), placeCombo);
    layout->addRow(tr("&Forecast:"), todayCheckBox);

    QVBoxLayout *mainLayout = new QVBoxLayout(this);
    mainLayout->addLayout(layout);
    mainLayout->addWidget(forecastTable);
    mainLayout->addWidget(creditLabel);

    setWindowTitle(tr("Weather Forecast"));
    fetchForecast("Italia/Toscana/Firenze");
}

void WeatherWidget::fetchForecast(const QString &place)
{
    uint now = QDateTime::currentDateTime().toTime_t();

    if (now - queryTimes[place] < 600) {
        receiver = recentQueries[place];
        updateTable();
        return;
    }

    QString url = QString("http://www.yr.no/place/%1/forecast.xml").arg(place);

    QXmlQuery query;
    query.bindVariable("url", QXmlItem(QVariant(url)));
    query.setQuery(query_string);

    if (query.isValid()) {
        receiver = new Receiver(query.namePool());
        query.evaluateTo(receiver);

        delete recentQueries[place];
        recentQueries[place] = receiver;
        queryTimes[place] = now;

        updateTable();
    }
}

void WeatherWidget::updateTable()
{
    if (receiver->sun.contains("rise") && receiver->sun.contains("set") &&
        receiver->forecasts.count() > 0) {

        forecastTable->clear();
        forecastTable->setRowCount(0);

        QDateTime sunrise = QDateTime::fromString(receiver->sun["rise"], Qt::ISODate);
        QDateTime sunset = QDateTime::fromString(receiver->sun["set"], Qt::ISODate);

        foreach (const Forecast &forecast, receiver->forecasts) {

            QDateTime datetime = QDateTime::fromString(forecast.time["from"], Qt::ISODate);
            bool onlyToday = todayCheckBox->checkState() == Qt::Checked;

            if (!onlyToday || datetime.date() == QDate::currentDate()) {

                QString description = forecast.symbol["name"];
                QString identifier = forecast.symbol["number"];
                if (identifier.count() == 1)
                    identifier.prepend("0");
                QString temperature = forecast.temperature["value"];
                QString unit = forecast.temperature["unit"];
                if (unit == "celcius")
                    unit = QChar(0xb0) + QString("C");

                int row = forecastTable->rowCount();
                forecastTable->insertRow(row);

                QTableWidgetItem *timeItem;
                if (onlyToday)
                    timeItem = new QTableWidgetItem(datetime.time().toString());
                else
                    timeItem = new QTableWidgetItem(datetime.toString("dddd\nhh:mm"));

                timeItem->setFlags(Qt::ItemIsEnabled);
                forecastTable->setItem(row, 0, timeItem);

                QString symbol;
                if (sunrise.time() <= datetime.time() && datetime.time() < sunset.time())
                    symbol = identifier + "d.png";
                else
                    symbol = identifier + "n.png";

                QIcon icon;
                if (QFile::exists(":images/"+symbol))
                    icon = QIcon(":images/"+symbol);
                else if (QFile::exists(":images/"+identifier+".png"))
                    icon = QIcon(":images/"+identifier+".png");

                QTableWidgetItem *symbolItem;
                if (!icon.isNull())
                    symbolItem = new QTableWidgetItem(icon, description);
                else
                    symbolItem = new QTableWidgetItem(description);

                symbolItem->setFlags(Qt::ItemIsEnabled);

                forecastTable->setItem(row, 1, symbolItem);

                QTableWidgetItem *temperatureItem = new QTableWidgetItem(temperature + " " + unit);
                temperatureItem->setFlags(Qt::ItemIsEnabled);
                forecastTable->setItem(row, 2, temperatureItem);
            }
        }

        forecastTable->resizeColumnsToContents();
        forecastTable->resizeRowsToContents();
    }
}
